@@ -1,6 +1,8 @@
 /*
  * Copyright (C) 2009-2011, Pino Toscano <pino@kde.org>
- * Copyright (C) 2016 Jakub Kucharski <jakubkucharski97@gmail.com>
+ * Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
+ * Copyright (C) 2017, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2018, Adam Reichold <adam.reichold@t-online.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +26,7 @@
 
 #include "poppler-document-private.h"
 #include "poppler-embedded-file-private.h"
+#include "poppler-page-private.h"
 #include "poppler-private.h"
 #include "poppler-toc-private.h"
 
@@ -45,7 +48,7 @@ initer::initer()
 {
     if (!count) {
         globalParams = new GlobalParams();
-        setErrorCallback(detail::error_function, NULL);
+        setErrorCallback(detail::error_function, nullptr);
     }
     count++;
 }
@@ -56,7 +59,7 @@ initer::~initer()
         --count;
         if (!count) {
             delete globalParams;
-            globalParams = 0;
+            globalParams = nullptr;
         }
     }
 }
@@ -65,8 +68,8 @@ initer::~initer()
 document_private::document_private(GooString *file_path, const std::string &owner_password,
                                    const std::string &user_password)
     : initer()
-    , doc(0)
-    , raw_doc_data(0)
+    , doc(nullptr)
+    , raw_doc_data(nullptr)
     , raw_doc_data_length(0)
     , is_locked(false)
 {
@@ -79,15 +82,13 @@ document_private::document_private(byte_array *file_data,
                                    const std::string &owner_password,
                                    const std::string &user_password)
     : initer()
-    , doc(0)
-    , raw_doc_data(0)
+    , doc(nullptr)
+    , raw_doc_data(nullptr)
     , raw_doc_data_length(0)
     , is_locked(false)
 {
-    Object obj;
-    obj.initNull();
     file_data->swap(doc_data);
-    MemStream *memstr = new MemStream(&doc_data[0], 0, doc_data.size(), &obj);
+    MemStream *memstr = new MemStream(&doc_data[0], 0, doc_data.size(), Object(objNull));
     GooString goo_owner_password(owner_password.c_str());
     GooString goo_user_password(user_password.c_str());
     doc = new PDFDoc(memstr, &goo_owner_password, &goo_user_password);
@@ -97,14 +98,12 @@ document_private::document_private(const char *file_data, int file_data_length,
                                    const std::string &owner_password,
                                    const std::string &user_password)
     : initer()
-    , doc(0)
+    , doc(nullptr)
     , raw_doc_data(file_data)
     , raw_doc_data_length(file_data_length)
     , is_locked(false)
 {
-    Object obj;
-    obj.initNull();
-    MemStream *memstr = new MemStream(const_cast<char *>(raw_doc_data), 0, raw_doc_data_length, &obj);
+    MemStream *memstr = new MemStream(const_cast<char *>(raw_doc_data), 0, raw_doc_data_length, Object(objNull));
     GooString goo_owner_password(owner_password.c_str());
     GooString goo_user_password(user_password.c_str());
     doc = new PDFDoc(memstr, &goo_owner_password, &goo_user_password);
@@ -131,7 +130,7 @@ document* document_private::check_document(document_private *doc, byte_array *fi
         }
         delete doc;
     }
-    return 0;
+    return nullptr;
 }
 
 /**
@@ -199,14 +198,14 @@ bool document::is_locked() const
 }
 
 /**
- Unlocks the current doocument, if locked.
+ Unlocks the current document, if locked.
 
  \returns the new locking status of the document
  */
 bool document::unlock(const std::string &owner_password, const std::string &user_password)
 {
     if (d->is_locked) {
-        document_private *newdoc = 0;
+        document_private *newdoc = nullptr;
         if (d->doc_data.size() > 0) {
             newdoc = new document_private(&d->doc_data,
                                           owner_password, user_password);
@@ -313,9 +312,8 @@ std::vector<std::string> document::info_keys() const
         return std::vector<std::string>();
     }
 
-    Object info;
-    if (!d->doc->getDocInfo(&info)->isDict()) {
-        info.free();
+    Object info = d->doc->getDocInfo();
+    if (!info.isDict()) {
         return std::vector<std::string>();
     }
 
@@ -325,7 +323,6 @@ std::vector<std::string> document::info_keys() const
         keys[i] = std::string(info_dict->getKey(i));
     }
 
-    info.free();
     return keys;
 }
 
@@ -341,7 +338,7 @@ ustring document::info_key(const std::string &key) const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_value(d->doc->getDocInfoStringEntry(key.c_str()));
+    std::unique_ptr<GooString> goo_value(d->doc->getDocInfoStringEntry(key.c_str()));
     if (!goo_value.get()) {
         return ustring();
     }
@@ -364,7 +361,7 @@ bool document::set_info_key(const std::string &key, const ustring &val)
     GooString *goo_val;
 
     if (val.empty()) {
-        goo_val = NULL;
+        goo_val = nullptr;
     } else {
         goo_val = detail::ustring_to_unicode_GooString(val);
     }
@@ -386,7 +383,7 @@ time_type document::info_date(const std::string &key) const
         return time_type(-1);
     }
 
-    std::auto_ptr<GooString> goo_date(d->doc->getDocInfoStringEntry(key.c_str()));
+    std::unique_ptr<GooString> goo_date(d->doc->getDocInfoStringEntry(key.c_str()));
     if (!goo_date.get()) {
         return time_type(-1);
     }
@@ -410,7 +407,7 @@ bool document::set_info_date(const std::string &key, time_type val)
     GooString *goo_date;
 
     if (val == time_type(-1)) {
-        goo_date = NULL;
+        goo_date = nullptr;
     } else {
         time_t t = static_cast<time_t> (val);
         goo_date = timeToDateString(&t);
@@ -432,7 +429,7 @@ ustring document::get_title() const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_title(d->doc->getDocInfoTitle());
+    std::unique_ptr<GooString> goo_title(d->doc->getDocInfoTitle());
     if (!goo_title.get()) {
         return ustring();
     }
@@ -455,7 +452,7 @@ bool document::set_title(const ustring &title)
     GooString *goo_title;
 
     if (title.empty()) {
-        goo_title = NULL;
+        goo_title = nullptr;
     } else {
         goo_title = detail::ustring_to_unicode_GooString(title);
     }
@@ -476,7 +473,7 @@ ustring document::get_author() const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_author(d->doc->getDocInfoAuthor());
+    std::unique_ptr<GooString> goo_author(d->doc->getDocInfoAuthor());
     if (!goo_author.get()) {
         return ustring();
     }
@@ -499,7 +496,7 @@ bool document::set_author(const ustring &author)
     GooString *goo_author;
 
     if (author.empty()) {
-        goo_author = NULL;
+        goo_author = nullptr;
     } else {
         goo_author = detail::ustring_to_unicode_GooString(author);
     }
@@ -520,7 +517,7 @@ ustring document::get_subject() const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_subject(d->doc->getDocInfoSubject());
+    std::unique_ptr<GooString> goo_subject(d->doc->getDocInfoSubject());
     if (!goo_subject.get()) {
         return ustring();
     }
@@ -543,7 +540,7 @@ bool document::set_subject(const ustring &subject)
     GooString *goo_subject;
 
     if (subject.empty()) {
-        goo_subject = NULL;
+        goo_subject = nullptr;
     } else {
         goo_subject = detail::ustring_to_unicode_GooString(subject);
     }
@@ -564,7 +561,7 @@ ustring document::get_keywords() const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_keywords(d->doc->getDocInfoKeywords());
+    std::unique_ptr<GooString> goo_keywords(d->doc->getDocInfoKeywords());
     if (!goo_keywords.get()) {
         return ustring();
     }
@@ -587,7 +584,7 @@ bool document::set_keywords(const ustring &keywords)
     GooString *goo_keywords;
 
     if (keywords.empty()) {
-        goo_keywords = NULL;
+        goo_keywords = nullptr;
     } else {
         goo_keywords = detail::ustring_to_unicode_GooString(keywords);
     }
@@ -608,7 +605,7 @@ ustring document::get_creator() const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_creator(d->doc->getDocInfoCreator());
+    std::unique_ptr<GooString> goo_creator(d->doc->getDocInfoCreator());
     if (!goo_creator.get()) {
         return ustring();
     }
@@ -631,7 +628,7 @@ bool document::set_creator(const ustring &creator)
     GooString *goo_creator;
 
     if (creator.empty()) {
-        goo_creator = NULL;
+        goo_creator = nullptr;
     } else {
         goo_creator = detail::ustring_to_unicode_GooString(creator);
     }
@@ -652,7 +649,7 @@ ustring document::get_producer() const
         return ustring();
     }
 
-    std::auto_ptr<GooString> goo_producer(d->doc->getDocInfoProducer());
+    std::unique_ptr<GooString> goo_producer(d->doc->getDocInfoProducer());
     if (!goo_producer.get()) {
         return ustring();
     }
@@ -675,7 +672,7 @@ bool document::set_producer(const ustring &producer)
     GooString *goo_producer;
 
     if (producer.empty()) {
-        goo_producer = NULL;
+        goo_producer = nullptr;
     } else {
         goo_producer = detail::ustring_to_unicode_GooString(producer);
     }
@@ -696,7 +693,7 @@ time_type document::get_creation_date() const
         return time_type(-1);
     }
 
-    std::auto_ptr<GooString> goo_creation_date(d->doc->getDocInfoCreatDate());
+    std::unique_ptr<GooString> goo_creation_date(d->doc->getDocInfoCreatDate());
     if (!goo_creation_date.get()) {
         return time_type(-1);
     }
@@ -719,7 +716,7 @@ bool document::set_creation_date(time_type creation_date)
     GooString *goo_creation_date;
 
     if (creation_date == time_type(-1)) {
-        goo_creation_date = NULL;
+        goo_creation_date = nullptr;
     } else {
         time_t t = static_cast<time_t> (creation_date);
         goo_creation_date = timeToDateString(&t);
@@ -741,7 +738,7 @@ time_type document::get_modification_date() const
         return time_type(-1);
     }
 
-    std::auto_ptr<GooString> goo_modification_date(d->doc->getDocInfoModDate());
+    std::unique_ptr<GooString> goo_modification_date(d->doc->getDocInfoModDate());
     if (!goo_modification_date.get()) {
         return time_type(-1);
     }
@@ -764,7 +761,7 @@ bool document::set_modification_date(time_type mod_date)
     GooString *goo_mod_date;
 
     if (mod_date == time_type(-1)) {
-        goo_mod_date = NULL;
+        goo_mod_date = nullptr;
     } else {
         time_t t = static_cast<time_t> (mod_date);
         goo_mod_date = timeToDateString(&t);
@@ -840,7 +837,7 @@ bool document::has_permission(permission_enum which) const
  */
 ustring document::metadata() const
 {
-    std::auto_ptr<GooString> md(d->doc->getCatalog()->readMetadata());
+    std::unique_ptr<GooString> md(d->doc->getCatalog()->readMetadata());
     if (md.get()) {
         return detail::unicode_GooString_to_ustring(md.get());
     }
@@ -862,7 +859,7 @@ bool document::get_pdf_id(std::string *permanent_id, std::string *update_id) con
     GooString goo_permanent_id;
     GooString goo_update_id;
 
-    if (!d->doc->getID(permanent_id ? &goo_permanent_id : 0, update_id ? &goo_update_id : 0)) {
+    if (!d->doc->getID(permanent_id ? &goo_permanent_id : nullptr, update_id ? &goo_update_id : nullptr)) {
         return false;
     }
 
@@ -896,11 +893,11 @@ int document::pages() const
  */
 page* document::create_page(const ustring &label) const
 {
-    std::auto_ptr<GooString> goolabel(detail::ustring_to_unicode_GooString(label));
+    std::unique_ptr<GooString> goolabel(detail::ustring_to_unicode_GooString(label));
     int index = 0;
 
     if (!d->doc->getCatalog()->labelToIndex(goolabel.get(), &index)) {
-        return 0;
+        return nullptr;
     }
     return create_page(index);
 }
@@ -915,7 +912,17 @@ page* document::create_page(const ustring &label) const
  */
 page* document::create_page(int index) const
 {
-    return index >= 0 && index < d->doc->getNumPages() ? new page(d, index) : 0;
+    if (index >= 0 && index < d->doc->getNumPages()) {
+        page *p = new page(d, index);
+        if (p->d->page) {
+            return p;
+        } else {
+            delete p;
+            return nullptr;
+        }
+    } else {
+        return nullptr;
+    }
 }
 
 /**
@@ -1041,7 +1048,7 @@ document* document::load_from_file(const std::string &file_name,
     document_private *doc = new document_private(
                                 new GooString(file_name.c_str()),
                                 owner_password, user_password);
-    return document_private::check_document(doc, 0);
+    return document_private::check_document(doc, nullptr);
 }
 
 /**
@@ -1059,7 +1066,7 @@ document* document::load_from_data(byte_array *file_data,
                                    const std::string &user_password)
 {
     if (!file_data || file_data->size() < 10) {
-        return 0;
+        return nullptr;
     }
 
     document_private *doc = new document_private(
@@ -1087,11 +1094,11 @@ document* document::load_from_raw_data(const char *file_data,
                                        const std::string &user_password)
 {
     if (!file_data || file_data_length < 10) {
-        return 0;
+        return nullptr;
     }
 
     document_private *doc = new document_private(
                                 file_data, file_data_length,
                                 owner_password, user_password);
-    return document_private::check_document(doc, 0);
+    return document_private::check_document(doc, nullptr);
 }
