@@ -6,21 +6,31 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2016 Albert Astals Cid <aacid@kde.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef FOFIENCODINGS_H
 #define FOFIENCODINGS_H
 
-#ifdef USE_GCC_PRAGMAS
-#pragma interface
-#endif
-
 #include "goo/gtypes.h"
 
 //------------------------------------------------------------------------
 // Type 1 and 1C font data
 //------------------------------------------------------------------------
 
-extern const char *fofiType1StandardEncoding[256];
-extern const char *fofiType1ExpertEncoding[256];
+extern const char * const fofiType1StandardEncoding[256];
+extern const char * const fofiType1ExpertEncoding[256];
 
 //------------------------------------------------------------------------
 // Type 1C font data
