@@ -17,7 +17,7 @@
 // Copyright (C) 2005-2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2005 Martin Kretzschmar <martink@gnome.org>
-// Copyright (C) 2005, 2009, 2012, 2013, 2015, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2009, 2012, 2013, 2015, 2017, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006, 2007, 2010, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2007 Koji Otani <sho@bbr.jp>
 // Copyright (C) 2008, 2009 Chris Wilson <chris@chris-wilson.co.uk>
@@ -28,6 +28,7 @@
 // Copyright (C) 2012 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2015, 2016 Jason Crain <jason@aquaticape.us>
+// Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -56,39 +57,29 @@
 #define CAN_CHECK_OPEN_FACES 1
 #endif
 
-#ifdef USE_GCC_PRAGMAS
-#pragma implementation
-#endif
-
-#ifdef MULTITHREADED
-#  define fontEngineLocker()   MutexLocker locker(&mutex)
-#else
-#  define fontEngineLocker()
-#endif
-
 //------------------------------------------------------------------------
 // CairoFont
 //------------------------------------------------------------------------
 
-CairoFont::CairoFont(Ref ref,
-		     cairo_font_face_t *cairo_font_face,
-		     int *codeToGID,
-		     Guint codeToGIDLen,
-		     GBool substitute,
-		     GBool printing) : ref(ref),
-				       cairo_font_face(cairo_font_face),
-				       codeToGID(codeToGID),
-				       codeToGIDLen(codeToGIDLen),
-				       substitute(substitute),
-				       printing(printing)      { }
+CairoFont::CairoFont(Ref refA,
+		     cairo_font_face_t *cairo_font_faceA,
+		     int *codeToGIDA,
+		     Guint codeToGIDLenA,
+		     bool substituteA,
+		     bool printingA) : ref(refA),
+				       cairo_font_face(cairo_font_faceA),
+				       codeToGID(codeToGIDA),
+				       codeToGIDLen(codeToGIDLenA),
+				       substitute(substituteA),
+				       printing(printingA)      { }
 
 CairoFont::~CairoFont() {
   cairo_font_face_destroy (cairo_font_face);
   gfree(codeToGID);
 }
 
-GBool
-CairoFont::matches(Ref &other, GBool printingA) {
+bool
+CairoFont::matches(Ref &other, bool printingA) {
   return (other.num == ref.num && other.gen == ref.gen);
 }
 
@@ -115,7 +106,7 @@ CairoFont::getSubstitutionCorrection(GfxFont *gfxFont)
 {
   double w1, w2, w3;
   CharCode code;
-  char *name;
+  const char *name;
 
   // for substituted fonts: adjust the font matrix -- compare the
   // width of 'm' in the original font and the substituted font
@@ -170,7 +161,7 @@ _ft_done_face_uncached (void *closure)
     FT_Done_Face (face);
 }
 
-static GBool
+static bool
 _ft_new_face_uncached (FT_Library lib,
 		       const char *filename,
 		       char *font_data,
@@ -181,12 +172,12 @@ _ft_new_face_uncached (FT_Library lib,
   FT_Face face;
   cairo_font_face_t *font_face;
 
-  if (font_data == NULL) {
+  if (font_data == nullptr) {
     if (FT_New_Face (lib, filename, 0, &face))
-      return gFalse;
+      return false;
   } else {
     if (FT_New_Memory_Face (lib, (unsigned char *)font_data, font_data_len, 0, &face))
-      return gFalse;
+      return false;
   }
 
   font_face = cairo_ft_font_face_create_for_ft_face (face,
@@ -199,12 +190,12 @@ _ft_new_face_uncached (FT_Library lib,
   {
     _ft_done_face_uncached (face);
     cairo_font_face_destroy (font_face);
-    return gFalse;
+    return false;
   }
 
   *face_out = face;
   *font_face_out = font_face;
-  return gTrue;
+  return true;
 }
 
 #ifdef CAN_CHECK_OPEN_FACES
@@ -233,15 +224,15 @@ _djb_hash (const unsigned char *bytes, size_t len)
   return hash;
 }
 
-static GBool
+static bool
 _ft_face_data_equal (struct _ft_face_data *a, struct _ft_face_data *b)
 {
   if (a->lib != b->lib)
-    return gFalse;
+    return false;
   if (a->size != b->size)
-    return gFalse;
+    return false;
   if (a->hash != b->hash)
-    return gFalse;
+    return false;
 
   return memcmp (a->bytes, b->bytes, a->size) == 0;
 }
@@ -273,7 +264,7 @@ _ft_done_face (void *closure)
   gfree (data);
 }
 
-static GBool
+static bool
 _ft_new_face (FT_Library lib,
 	      const char *filename,
 	      char *font_data,
@@ -287,7 +278,7 @@ _ft_new_face (FT_Library lib,
 
   tmpl.fd = -1;
 
-  if (font_data == NULL) {
+  if (font_data == nullptr) {
     /* if we fail to mmap the file, just pass it to FreeType instead */
     tmpl.fd = open (filename, O_RDONLY);
     if (tmpl.fd == -1)
@@ -298,7 +289,7 @@ _ft_new_face (FT_Library lib,
       return _ft_new_face_uncached (lib, filename, font_data, font_data_len, face_out, font_face_out);
     }
 
-    tmpl.bytes = (unsigned char *) mmap (NULL, st.st_size,
+    tmpl.bytes = (unsigned char *) mmap (nullptr, st.st_size,
 					 PROT_READ, MAP_PRIVATE,
 					 tmpl.fd, 0);
     if (tmpl.bytes == MAP_FAILED) {
@@ -329,7 +320,7 @@ _ft_new_face (FT_Library lib,
       }
       *face_out = l->face;
       *font_face_out = cairo_font_face_reference (l->font_face);
-      return gTrue;
+      return true;
     }
   }
 
@@ -347,12 +338,12 @@ _ft_new_face (FT_Library lib,
 
       close (tmpl.fd);
     }
-    return gFalse;
+    return false;
   }
 
   l = (struct _ft_face_data *) gmallocn (1, sizeof (struct _ft_face_data));
   *l = tmpl;
-  l->prev = NULL;
+  l->prev = nullptr;
   l->next = _ft_open_faces;
   if (_ft_open_faces)
     _ft_open_faces->prev = l;
@@ -368,35 +359,34 @@ _ft_new_face (FT_Library lib,
   {
     cairo_font_face_destroy (l->font_face);
     _ft_done_face (l);
-    return gFalse;
+    return false;
   }
 
   *face_out = l->face;
   *font_face_out = l->font_face;
-  return gTrue;
+  return true;
 }
 #else
 #define _ft_new_face _ft_new_face_uncached
 #endif
 
-CairoFreeTypeFont::CairoFreeTypeFont(Ref ref,
-				     cairo_font_face_t *cairo_font_face,
-				     int *codeToGID,
-				     Guint codeToGIDLen,
-				     GBool substitute) : CairoFont(ref,
-								   cairo_font_face,
-								   codeToGID,
-								   codeToGIDLen,
-								   substitute,
-								   gTrue) { }
+CairoFreeTypeFont::CairoFreeTypeFont(Ref refA,
+				     cairo_font_face_t *cairo_font_faceA,
+				     int *codeToGIDA,
+				     Guint codeToGIDLenA,
+				     bool substituteA) : CairoFont(refA,
+								   cairo_font_faceA,
+								   codeToGIDA,
+								   codeToGIDLenA,
+								   substituteA,
+								   true) { }
 
 CairoFreeTypeFont::~CairoFreeTypeFont() { }
 
 CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
-					     FT_Library lib, GBool useCIDs) {
-  Object refObj, strObj;
+					     FT_Library lib, bool useCIDs) {
   GooString *fileName;
-  char *fileNameC;
+  const char *fileNameC;
   char *font_data;
   int font_data_len;
   int i, n;
@@ -413,19 +403,19 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
   int *codeToGID;
   Guint codeToGIDLen;
   
-  codeToGID = NULL;
+  codeToGID = nullptr;
   codeToGIDLen = 0;
-  font_data = NULL;
+  font_data = nullptr;
   font_data_len = 0;
-  fileName = NULL;
-  fileNameC = NULL;
+  fileName = nullptr;
+  fileNameC = nullptr;
 
-  GBool substitute = gFalse;
+  bool substitute = false;
   
   ref = *gfxFont->getID();
   fontType = gfxFont->getType();
 
-  if (!(fontLoc = gfxFont->locateFont(xref, NULL))) {
+  if (!(fontLoc = gfxFont->locateFont(xref, nullptr))) {
     error(errSyntaxError, -1, "Couldn't find a font for '{0:s}'",
     	gfxFont->getName() ? gfxFont->getName()->getCString()
 	                       : "(unnamed)");
@@ -435,17 +425,17 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
   // embedded font
   if (fontLoc->locType == gfxFontLocEmbedded) {
     font_data = gfxFont->readEmbFontFile(xref, &font_data_len);
-    if (NULL == font_data)
+    if (nullptr == font_data)
       goto err2;
 
   // external font
   } else { // gfxFontLocExternal
     fileName = fontLoc->path;
     fontType = fontLoc->fontType;
-    substitute = gTrue;
+    substitute = true;
   }
 
-  if (fileName != NULL) {
+  if (fileName != nullptr) {
     fileNameC = fileName->getCString();
   }
 
@@ -482,7 +472,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
     break;
   case fontCIDType2:
   case fontCIDType2OT:
-    codeToGID = NULL;
+    codeToGID = nullptr;
     n = 0;
     if (((GfxCIDFont *)gfxFont)->getCIDToGID()) {
       n = ((GfxCIDFont *)gfxFont)->getCIDToGIDLen();
@@ -492,7 +482,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
 		n * sizeof(int));
       }
     } else {
-      if (font_data != NULL) {
+      if (font_data != nullptr) {
         ff = FoFiTrueType::make(font_data, font_data_len);
       } else {
         ff = FoFiTrueType::load(fileNameC);
@@ -506,7 +496,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
     /* Fall through */
   case fontTrueType:
   case fontTrueTypeOT:
-    if (font_data != NULL) {
+    if (font_data != nullptr) {
       ff = FoFiTrueType::make(font_data, font_data_len);
     } else {
       ff = FoFiTrueType::load(fileNameC);
@@ -530,12 +520,12 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
   case fontCIDType0:
   case fontCIDType0C:
 
-    codeToGID = NULL;
+    codeToGID = nullptr;
     codeToGIDLen = 0;
 
     if (!useCIDs)
     {
-      if (font_data != NULL) {
+      if (font_data != nullptr) {
         ff1c = FoFiType1C::make(font_data, font_data_len);
       } else {
         ff1c = FoFiType1C::load(fileNameC);
@@ -553,7 +543,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
     break;
 
   case fontCIDType0COT:
-    codeToGID = NULL;
+    codeToGID = nullptr;
     n = 0;
     if (((GfxCIDFont *)gfxFont)->getCIDToGID()) {
       n = ((GfxCIDFont *)gfxFont)->getCIDToGIDLen();
@@ -567,7 +557,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
 
     if (!codeToGID) {
       if (!useCIDs) {
-	if (font_data != NULL) {
+	if (font_data != nullptr) {
 	  ff = FoFiTrueType::make(font_data, font_data_len);
 	} else {
 	  ff = FoFiTrueType::load(fileNameC);
@@ -604,7 +594,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(GfxFont *gfxFont, XRef *xref,
   gfree (codeToGID);
   gfree (font_data);
   fprintf (stderr, "some font thing failed\n");
-  return NULL;
+  return nullptr;
 }
 
 //------------------------------------------------------------------------
@@ -617,7 +607,7 @@ typedef struct _type3_font_info {
   GfxFont *font;
   PDFDoc *doc;
   CairoFontEngine *fontEngine;
-  GBool printing;
+  bool printing;
   XRef *xref;
 } type3_font_info_t;
 
@@ -637,13 +627,12 @@ _init_type3_glyph (cairo_scaled_font_t  *scaled_font,
 {
   type3_font_info_t *info;
   GfxFont *font;
-  double *mat;
 
   info = (type3_font_info_t *)
     cairo_font_face_get_user_data (cairo_scaled_font_get_font_face (scaled_font),
 				   &type3_font_key);
   font = info->font;
-  mat = font->getFontBBox();
+  const double *mat = font->getFontBBox();
   extents->ascent = mat[3]; /* y2 */
   extents->descent = -mat[3]; /* -y1 */
   extents->height = extents->ascent + extents->descent;
@@ -663,7 +652,7 @@ _render_type3_glyph (cairo_scaled_font_t  *scaled_font,
   Object charProc;
   CairoOutputDev *output_dev;
   cairo_matrix_t matrix, invert_y_axis;
-  double *mat;
+  const double *mat;
   double wx, wy;
   PDFRectangle box;
   type3_font_info_t *info;
@@ -704,10 +693,10 @@ _render_type3_glyph (cairo_scaled_font_t  *scaled_font,
   box.y1 = mat[1];
   box.x2 = mat[2];
   box.y2 = mat[3];
-  gfx = new Gfx(info->doc, output_dev, resDict, &box, NULL);
+  gfx = new Gfx(info->doc, output_dev, resDict, &box, nullptr);
   output_dev->startDoc(info->doc, info->fontEngine);
   output_dev->startPage (1, gfx->getState(), gfx->getXRef());
-  output_dev->setInType3Char(gTrue);
+  output_dev->setInType3Char(true);
   charProc = charProcs->getVal(glyph);
   gfx->display(&charProc);
 
@@ -735,8 +724,7 @@ _render_type3_glyph (cairo_scaled_font_t  *scaled_font,
 
 CairoType3Font *CairoType3Font::create(GfxFont *gfxFont, PDFDoc *doc,
 				       CairoFontEngine *fontEngine,
-				       GBool printing, XRef *xref) {
-  Object refObj, strObj;
+				       bool printing, XRef *xref) {
   type3_font_info_t *info;
   cairo_font_face_t *font_face;
   Ref ref;
@@ -776,26 +764,25 @@ CairoType3Font *CairoType3Font::create(GfxFont *gfxFont, PDFDoc *doc,
     }
   }
 
-  return new CairoType3Font(ref, doc, font_face, codeToGID, codeToGIDLen, printing, xref);
+  return new CairoType3Font(ref, font_face, codeToGID, codeToGIDLen, printing, xref);
 }
 
-CairoType3Font::CairoType3Font(Ref ref,
-			       PDFDoc *doc,
-			       cairo_font_face_t *cairo_font_face,
-			       int *codeToGID,
-			       Guint codeToGIDLen,
-			       GBool printing, XRef *xref) : CairoFont(ref,
-							   cairo_font_face,
-							   codeToGID,
-							   codeToGIDLen,
-							   gFalse,
-							   printing),
-						 doc(doc) { }
+CairoType3Font::CairoType3Font(Ref refA,
+			       cairo_font_face_t *cairo_font_faceA,
+			       int *codeToGIDA,
+			       Guint codeToGIDLenA,
+			       bool printingA, XRef *xref) : CairoFont(refA,
+							   cairo_font_faceA,
+							   codeToGIDA,
+							   codeToGIDLenA,
+							   false,
+							   printingA)
+{ }
 
 CairoType3Font::~CairoType3Font() { }
 
-GBool
-CairoType3Font::matches(Ref &other, GBool printingA) {
+bool
+CairoType3Font::matches(Ref &other, bool printingA) {
   return (other.num == ref.num && other.gen == ref.gen && printing == printingA);
 }
 
@@ -804,12 +791,14 @@ CairoType3Font::matches(Ref &other, GBool printingA) {
 // CairoFontEngine
 //------------------------------------------------------------------------
 
+#define fontEngineLocker()   std::unique_lock<std::recursive_mutex> locker(mutex)
+
 CairoFontEngine::CairoFontEngine(FT_Library libA) {
   int i;
 
   lib = libA;
   for (i = 0; i < cairoFontCacheSize; ++i) {
-    fontCache[i] = NULL;
+    fontCache[i] = nullptr;
   }
   
   FT_Int major, minor, patch;
@@ -817,9 +806,6 @@ CairoFontEngine::CairoFontEngine(FT_Library libA) {
   FT_Library_Version(lib, &major, &minor, &patch);
   useCIDs = major > 2 ||
             (major == 2 && (minor > 1 || (minor == 1 && patch > 7)));
-#ifdef MULTITHREADED
-  gInitMutex(&mutex);
-#endif
 }
 
 CairoFontEngine::~CairoFontEngine() {
@@ -829,13 +815,10 @@ CairoFontEngine::~CairoFontEngine() {
     if (fontCache[i])
       delete fontCache[i];
   }
-#ifdef MULTITHREADED
-  gDestroyMutex(&mutex);
-#endif
 }
 
 CairoFont *
-CairoFontEngine::getFont(GfxFont *gfxFont, PDFDoc *doc, GBool printing, XRef *xref) {
+CairoFontEngine::getFont(GfxFont *gfxFont, PDFDoc *doc, bool printing, XRef *xref) {
   int i, j;
   Ref ref;
   CairoFont *font;
