@@ -17,11 +17,13 @@
 // Copyright (C) 2007 Iñigo Martínez <inigomartinez@gmail.com>
 // Copyright (C) 2008 Brad Hards <bradh@kde.org>
 // Copyright (C) 2008, 2010 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright (C) 2009-2013, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009-2013, 2017, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009, 2010, 2012, 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2010 David Benjamin <davidben@mit.edu>
 // Copyright (C) 2010 Christian Feuersänger <cfeuersaenger@googlemail.com>
 // Copyright (C) 2013 Fabio D'Urso <fabiodurso@hotmail.it>
+// Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+// Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -31,10 +33,6 @@
 #ifndef GFX_H
 #define GFX_H
 
-#ifdef USE_GCC_PRAGMAS
-#pragma interface
-#endif
-
 #include "poppler-config.h"
 #include "goo/gtypes.h"
 #include "goo/GooList.h"
@@ -112,19 +110,24 @@ public:
   GfxResources(XRef *xref, Dict *resDict, GfxResources *nextA);
   ~GfxResources();
 
-  GfxFont *lookupFont(char *name);
-  Object lookupXObject(char *name);
-  Object lookupXObjectNF(char *name);
-  Object lookupMarkedContentNF(char *name);
-  Object lookupColorSpace(const char *name);
-  GfxPattern *lookupPattern(char *name, OutputDev *out, GfxState *state);
-  GfxShading *lookupShading(char *name, OutputDev *out, GfxState *state);
-  Object lookupGState(char *name);
-  Object lookupGStateNF(char *name);
+  GfxResources(const GfxResources &) = delete;
+  GfxResources& operator=(const GfxResources &other) = delete;
 
-  GfxResources *getNext() { return next; }
+  GfxFont *lookupFont(const char *name);
+  const GfxFont *lookupFont(const char *name) const;
+  Object lookupXObject(const char *name);
+  Object lookupXObjectNF(const char *name);
+  Object lookupMarkedContentNF(const char *name);
+  Object lookupColorSpace(const char *name);
+  GfxPattern *lookupPattern(const char *name, OutputDev *out, GfxState *state);
+  GfxShading *lookupShading(const char *name, OutputDev *out, GfxState *state);
+  Object lookupGState(const char *name);
+  Object lookupGStateNF(const char *name);
+
+  GfxResources *getNext() const { return next; }
 
 private:
+  GfxFont *doLookupFont(const char *name) const;
 
   GfxFontDict *fonts;
   Object xObjDict;
@@ -132,7 +135,8 @@ private:
   Object patternDict;
   Object shadingDict;
   Object gStateDict;
-  PopplerObjectCache gStateCache;
+  PopplerCache<Ref, Object> gStateCache;
+  XRef *xref;
   Object propertiesDict;
   GfxResources *next;
 };
@@ -146,25 +150,28 @@ public:
 
   // Constructor for regular output.
   Gfx(PDFDoc *docA, OutputDev *outA, int pageNum, Dict *resDict,
-      double hDPI, double vDPI, PDFRectangle *box,
-      PDFRectangle *cropBox, int rotate,
-      GBool (*abortCheckCbkA)(void *data) = NULL,
-      void *abortCheckCbkDataA = NULL, XRef *xrefA = NULL);
+      double hDPI, double vDPI, const PDFRectangle *box,
+      const PDFRectangle *cropBox, int rotate,
+      bool (*abortCheckCbkA)(void *data) = nullptr,
+      void *abortCheckCbkDataA = nullptr, XRef *xrefA = nullptr);
 
   // Constructor for a sub-page object.
   Gfx(PDFDoc *docA, OutputDev *outA, Dict *resDict,
-      PDFRectangle *box, PDFRectangle *cropBox,
-      GBool (*abortCheckCbkA)(void *data) = NULL,
-      void *abortCheckCbkDataA = NULL, XRef *xrefA = NULL);
+      const PDFRectangle *box, const PDFRectangle *cropBox,
+      bool (*abortCheckCbkA)(void *data) = nullptr,
+      void *abortCheckCbkDataA = nullptr, Gfx *gfxA = nullptr);
 #ifdef USE_CMS
   void initDisplayProfile();
 #endif
   ~Gfx();
 
+  Gfx(const Gfx &) = delete;
+  Gfx& operator=(const Gfx &other) = delete;
+
   XRef *getXRef() { return xref; }
 
   // Interpret a stream or array of streams.
-  void display(Object *obj, GBool topLevel = gTrue);
+  void display(Object *obj, bool topLevel = true);
 
   // Display an annotation, given its appearance (a Form XObject),
   // border style, and bounding box (in default user space).
@@ -186,14 +193,14 @@ public:
   // Get the current graphics state object.
   GfxState *getState() { return state; }
 
-  GBool checkTransparencyGroup(Dict *resDict);
+  bool checkTransparencyGroup(Dict *resDict);
 
-  void drawForm(Object *str, Dict *resDict, double *matrix, double *bbox,
-	       GBool transpGroup = gFalse, GBool softMask = gFalse,
-	       GfxColorSpace *blendingColorSpace = NULL,
-	       GBool isolated = gFalse, GBool knockout = gFalse,
-	       GBool alpha = gFalse, Function *transferFunc = NULL,
-	       GfxColor *backdropColor = NULL);
+  void drawForm(Object *str, Dict *resDict, const double *matrix, const double *bbox,
+	       bool transpGroup = false, bool softMask = false,
+	       GfxColorSpace *blendingColorSpace = nullptr,
+	       bool isolated = false, bool knockout = false,
+	       bool alpha = false, Function *transferFunc = nullptr,
+	       GfxColor *backdropColor = nullptr);
 
   void pushResources(Dict *resDict);
   void popResources();
@@ -204,42 +211,42 @@ private:
   XRef *xref;			// the xref table for this PDF file
   Catalog *catalog;		// the Catalog for this PDF file  
   OutputDev *out;		// output device
-  GBool subPage;		// is this a sub-page object?
-  GBool printCommands;		// print the drawing commands (for debugging)
-  GBool profileCommands;	// profile the drawing commands (for debugging)
-  GBool commandAborted;         // did the previous command abort the drawing?
+  bool subPage;		// is this a sub-page object?
+  bool printCommands;		// print the drawing commands (for debugging)
+  bool profileCommands;	// profile the drawing commands (for debugging)
+  bool commandAborted;         // did the previous command abort the drawing?
   GfxResources *res;		// resource stack
   int updateLevel;
 
   GfxState *state;		// current graphics state
   int stackHeight;		// the height of the current graphics stack
   std::vector<int> stateGuards;   // a stack of state limits; to guard against unmatched pops
-  GBool fontChanged;		// set if font or text matrix has changed
+  bool fontChanged;		// set if font or text matrix has changed
   GfxClipType clip;		// do a clip?
   int ignoreUndef;		// current BX/EX nesting level
   double baseMatrix[6];		// default matrix for most recent
 				//   page/form/pattern
   int formDepth;
-  GBool ocState;		// true if drawing is enabled, false if
+  bool ocState;		// true if drawing is enabled, false if
 				//   disabled
 
   MarkedContentStack *mcStack;	// current BMC/EMC stack
 
   Parser *parser;		// parser for page content stream(s)
   
-  std::set<int> formsDrawing;	// the forms that are being drawn
+  std::set<int> formsDrawing;	// the forms/patterns that are being drawn
   std::set<int> charProcDrawing;	// the charProc that are being drawn
 
-  GBool				// callback to check for an abort
+  bool				// callback to check for an abort
     (*abortCheckCbk)(void *data);
   void *abortCheckCbkData;
 
   static Operator opTab[];	// table of operators
 
-  void go(GBool topLevel);
+  void go(bool topLevel);
   void execOp(Object *cmd, Object args[], int numArgs);
-  Operator *findOp(char *name);
-  GBool checkArg(Object *arg, TchkType type);
+  Operator *findOp(const char *name);
+  bool checkArg(Object *arg, TchkType type);
   Goffset getPos();
 
   int bottomGuard();
@@ -255,9 +262,9 @@ private:
   void opSetMiterLimit(Object args[], int numArgs);
   void opSetLineWidth(Object args[], int numArgs);
   void opSetExtGState(Object args[], int numArgs);
-  void doSoftMask(Object *str, GBool alpha,
+  void doSoftMask(Object *str, bool alpha,
 		  GfxColorSpace *blendingColorSpace,
-		  GBool isolated, GBool knockout,
+		  bool isolated, bool knockout,
 		  Function *transferFunc, GfxColor *backdropColor);
   void opSetRenderingIntent(Object args[], int numArgs);
 
@@ -294,15 +301,15 @@ private:
   void opCloseFillStroke(Object args[], int numArgs);
   void opEOFillStroke(Object args[], int numArgs);
   void opCloseEOFillStroke(Object args[], int numArgs);
-  void doPatternFill(GBool eoFill);
+  void doPatternFill(bool eoFill);
   void doPatternStroke();
   void doPatternText();
   void doPatternImageMask(Object *ref, Stream *str, int width, int height,
-			  GBool invert, GBool inlineImg);
+			  bool invert, bool inlineImg);
   void doTilingPatternFill(GfxTilingPattern *tPat,
-			   GBool stroke, GBool eoFill, GBool text);
+			   bool stroke, bool eoFill, bool text);
   void doShadingPatternFill(GfxShadingPattern *sPat,
-			    GBool stroke, GBool eoFill, GBool text);
+			    bool stroke, bool eoFill, bool text);
   void opShFill(Object args[], int numArgs);
   void doFunctionShFill(GfxFunctionShading *shading);
   void doFunctionShFill1(GfxFunctionShading *shading,
@@ -321,7 +328,7 @@ private:
 			   double x2, double y2, double color2,
 			   double refineColorThreshold, int depth, GfxGouraudTriangleShading *shading, GfxState::ReusablePathIterator *path);
   void doPatchMeshShFill(GfxPatchMeshShading *shading);
-  void fillPatch(GfxPatch *patch, int colorComps, int patchColorComps, double refineColorThreshold, int depth, GfxPatchMeshShading *shading);
+  void fillPatch(const GfxPatch *patch, int colorComps, int patchColorComps, double refineColorThreshold, int depth, const GfxPatchMeshShading *shading);
   void doEndPath();
 
   // path clipping operators
@@ -352,12 +359,12 @@ private:
   void opMoveShowText(Object args[], int numArgs);
   void opMoveSetShowText(Object args[], int numArgs);
   void opShowSpaceText(Object args[], int numArgs);
-  void doShowText(GooString *s);
-  void doIncCharCount(GooString *s);
+  void doShowText(const GooString *s);
+  void doIncCharCount(const GooString *s);
 
   // XObject operators
   void opXObject(Object args[], int numArgs);
-  void doImage(Object *ref, Stream *str, GBool inlineImg);
+  void doImage(Object *ref, Stream *str, bool inlineImg);
   void doForm(Object *str);
 
   // in-line image operators
@@ -380,7 +387,7 @@ private:
   void opMarkPoint(Object args[], int numArgs);
   GfxState *saveStateStack();
   void restoreStateStack(GfxState *oldState);
-  GBool contentIsHidden();
+  bool contentIsHidden();
   void pushMarkedContent();
   void popMarkedContent();
 };
