@@ -29,10 +29,6 @@
 
 #include <config.h>
 
-#ifdef USE_GCC_PRAGMAS
-#pragma implementation
-#endif
-
 #include <stddef.h>
 #include "Object.h"
 #include "Array.h"
@@ -47,7 +43,7 @@
 // lots of nested arrays that made us consume all the stack
 #define recursionLimit 500
 
-Parser::Parser(XRef *xrefA, Lexer *lexerA, GBool allowStreamsA) {
+Parser::Parser(XRef *xrefA, Lexer *lexerA, bool allowStreamsA) {
   xref = xrefA;
   lexer = lexerA;
   inlineImg = 0;
@@ -62,14 +58,14 @@ Parser::~Parser() {
 
 Object Parser::getObj(int recursion)
 {
-  return getObj(gFalse, nullptr, cryptRC4, 0, 0, 0, recursion);
+  return getObj(false, nullptr, cryptRC4, 0, 0, 0, recursion);
 }
 
-Object Parser::getObj(GBool simpleOnly,
+Object Parser::getObj(bool simpleOnly,
            Guchar *fileKey,
 		       CryptAlgorithm encAlgorithm, int keyLength,
 		       int objNum, int objGen, int recursion,
-		       GBool strict) {
+		       bool strict) {
   Object obj;
   Stream *str;
   DecryptStream *decrypt;
@@ -93,7 +89,7 @@ Object Parser::getObj(GBool simpleOnly,
     shift();
     obj = Object(new Array(xref));
     while (!buf1.isCmd("]") && !buf1.isEOF() && recursion + 1 < recursionLimit) {
-      Object obj2 = getObj(gFalse, fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1);
+      Object obj2 = getObj(false, fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1);
       obj.arrayAdd(std::move(obj2));
     }
     if (recursion + 1 >= recursionLimit && strict) goto err;
@@ -120,7 +116,7 @@ Object Parser::getObj(GBool simpleOnly,
 	  if (strict && buf1.isError()) goto err;
 	  break;
 	}
-	Object obj2 = getObj(gFalse, fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1);
+	Object obj2 = getObj(false, fileKey, encAlgorithm, keyLength, objNum, objGen, recursion + 1);
 	if (unlikely(obj2.isError() && recursion + 1 >= recursionLimit)) {
 	  break;
 	}
@@ -196,7 +192,7 @@ err:
 Stream *Parser::makeStream(Object &&dict, Guchar *fileKey,
 			   CryptAlgorithm encAlgorithm, int keyLength,
 			   int objNum, int objGen, int recursion,
-                           GBool strict) {
+                           bool strict) {
   BaseStream *baseStr;
   Stream *str;
   Goffset length;
@@ -265,13 +261,13 @@ Stream *Parser::makeStream(Object &&dict, Guchar *fileKey,
       // When building the xref we can't use it so use this
       // kludge for broken PDF files: just add 5k to the length, and
       // hope its enough
-      if (length < LLONG_MAX - 5000)
+      if (length < LLONG_MAX - pos - 5000)
         length += 5000;
     }
   }
 
   // make base stream
-  str = baseStr->makeSubStream(pos, gTrue, length, std::move(dict));
+  str = baseStr->makeSubStream(pos, true, length, std::move(dict));
 
   // handle decryption
   if (fileKey) {
