@@ -13,10 +13,6 @@
 //
 //========================================================================
 
-#ifdef USE_GCC_PRAGMAS
-#pragma interface
-#endif
-
 #include "goo/GooString.h"
 #include "StructTreeRoot.h"
 #include "StructElement.h"
@@ -62,7 +58,7 @@ void StructTreeRoot::parse(Dict *root)
   std::set<int> seenElements;
 
   // Parse the children StructElements
-  const GBool marked = doc->getCatalog()->getMarkInfo() & Catalog::markInfoMarked;
+  const bool marked = doc->getCatalog()->getMarkInfo() & Catalog::markInfoMarked;
   Object kids = root->lookup("K");
   if (kids.isArray()) {
     if (marked && kids.arrayGetLength() > 1) {
