@@ -21,10 +21,6 @@
 
 #include <config.h>
 
-#ifdef USE_GCC_PRAGMAS
-#pragma implementation
-#endif
-
 #include <string.h>
 #include "goo/gmem.h"
 #include "SplashPattern.h"
@@ -45,7 +41,7 @@ int splashColorModeNComps[] = {
 #endif
 };
 
-SplashState::SplashState(int width, int height, GBool vectorAntialias,
+SplashState::SplashState(int width, int height, bool vectorAntialias,
 			 SplashScreenParams *screenParams) {
   SplashColor color;
   int i;
@@ -57,10 +53,10 @@ SplashState::SplashState(int width, int height, GBool vectorAntialias,
   strokePattern = new SplashSolidColor(color);
   fillPattern = new SplashSolidColor(color);
   screen = new SplashScreen(screenParams);
-  blendFunc = NULL;
+  blendFunc = nullptr;
   strokeAlpha = 1;
   fillAlpha = 1;
-  multiplyPatternAlpha = gFalse;
+  multiplyPatternAlpha = false;
   patternStrokeAlpha = 1;
   patternFillAlpha = 1;
   lineWidth = 0;
@@ -68,16 +64,16 @@ SplashState::SplashState(int width, int height, GBool vectorAntialias,
   lineJoin = splashLineJoinMiter;
   miterLimit = 10;
   flatness = 1;
-  lineDash = NULL;
+  lineDash = nullptr;
   lineDashLength = 0;
   lineDashPhase = 0;
-  strokeAdjust = gFalse;
+  strokeAdjust = false;
   clip = new SplashClip(0, 0, width - 0.001, height - 0.001, vectorAntialias);
-  softMask = NULL;
-  deleteSoftMask = gFalse;
-  inNonIsolatedGroup = gFalse;
-  fillOverprint = gFalse;
-  strokeOverprint = gFalse;
+  softMask = nullptr;
+  deleteSoftMask = false;
+  inNonIsolatedGroup = false;
+  fillOverprint = false;
+  strokeOverprint = false;
   overprintMode = 0;	  
   for (i = 0; i < 256; ++i) {
     rgbTransferR[i] = (Guchar)i;
@@ -94,11 +90,11 @@ SplashState::SplashState(int width, int height, GBool vectorAntialias,
 #endif
   }
   overprintMask = 0xffffffff;
-  overprintAdditive = gFalse;
-  next = NULL;
+  overprintAdditive = false;
+  next = nullptr;
 }
 
-SplashState::SplashState(int width, int height, GBool vectorAntialias,
+SplashState::SplashState(int width, int height, bool vectorAntialias,
 			 SplashScreen *screenA) {
   SplashColor color;
   int i;
@@ -110,10 +106,10 @@ SplashState::SplashState(int width, int height, GBool vectorAntialias,
   strokePattern = new SplashSolidColor(color);
   fillPattern = new SplashSolidColor(color);
   screen = screenA->copy();
-  blendFunc = NULL;
+  blendFunc = nullptr;
   strokeAlpha = 1;
   fillAlpha = 1;
-  multiplyPatternAlpha = gFalse;
+  multiplyPatternAlpha = false;
   patternStrokeAlpha = 1;
   patternFillAlpha = 1;
   lineWidth = 0;
@@ -121,16 +117,16 @@ SplashState::SplashState(int width, int height, GBool vectorAntialias,
   lineJoin = splashLineJoinMiter;
   miterLimit = 10;
   flatness = 1;
-  lineDash = NULL;
+  lineDash = nullptr;
   lineDashLength = 0;
   lineDashPhase = 0;
-  strokeAdjust = gFalse;
+  strokeAdjust = false;
   clip = new SplashClip(0, 0, width - 0.001, height - 0.001, vectorAntialias);
-  softMask = NULL;
-  deleteSoftMask = gFalse;
-  inNonIsolatedGroup = gFalse;
-  fillOverprint = gFalse;
-  strokeOverprint = gFalse;
+  softMask = nullptr;
+  deleteSoftMask = false;
+  inNonIsolatedGroup = false;
+  fillOverprint = false;
+  strokeOverprint = false;
   overprintMode = 0;	  
   for (i = 0; i < 256; ++i) {
     rgbTransferR[i] = (Guchar)i;
@@ -147,8 +143,8 @@ SplashState::SplashState(int width, int height, GBool vectorAntialias,
 #endif
   }
   overprintMask = 0xffffffff;
-  overprintAdditive = gFalse;
-  next = NULL;
+  overprintAdditive = false;
+  next = nullptr;
 }
 
 SplashState::SplashState(SplashState *state) {
@@ -172,14 +168,14 @@ SplashState::SplashState(SplashState *state) {
     lineDash = (SplashCoord *)gmallocn(lineDashLength, sizeof(SplashCoord));
     memcpy(lineDash, state->lineDash, lineDashLength * sizeof(SplashCoord));
   } else {
-    lineDash = NULL;
+    lineDash = nullptr;
     lineDashLength = 0;
   }
   lineDashPhase = state->lineDashPhase;
   strokeAdjust = state->strokeAdjust;
   clip = state->clip->copy();
   softMask = state->softMask;
-  deleteSoftMask = gFalse;
+  deleteSoftMask = false;
   inNonIsolatedGroup = state->inNonIsolatedGroup;
   fillOverprint = state->fillOverprint;
   strokeOverprint = state->strokeOverprint;
@@ -198,7 +194,7 @@ SplashState::SplashState(SplashState *state) {
 #endif
   overprintMask = state->overprintMask;
   overprintAdditive = state->overprintAdditive;
-  next = NULL;
+  next = nullptr;
 }
 
 SplashState::~SplashState() {
@@ -235,7 +231,7 @@ void SplashState::setLineDash(SplashCoord *lineDashA, int lineDashLengthA,
     lineDash = (SplashCoord *)gmallocn(lineDashLength, sizeof(SplashCoord));
     memcpy(lineDash, lineDashA, lineDashLength * sizeof(SplashCoord));
   } else {
-    lineDash = NULL;
+    lineDash = nullptr;
   }
   lineDashPhase = lineDashPhaseA;
 }
@@ -245,7 +241,7 @@ void SplashState::setSoftMask(SplashBitmap *softMaskA) {
     delete softMask;
   }
   softMask = softMaskA;
-  deleteSoftMask = gTrue;
+  deleteSoftMask = true;
 }
 
 void SplashState::setTransfer(Guchar *red, Guchar *green, Guchar *blue,
